/*
*	easyCrop v0.1
*	Author: s9wood
*	Description: Attempt to logically resize and crop an image to make none-proportional
*	images display more effectively relative to their container element.
*/

$.fn.easyCrop = function(options) {

	// Default settings.
	var settings = $.extend({
		onResize: false
	}, options );

	var increment = 0;

	return this.each(function() {
		// The image - container pair.
		var $image = $(this);
		var name = $image.attr('src');
		var $container = $(this).parents('.easyContainer');

		// Add class to each container for easy selection incase image needs
		// manually adjusting at specific resolutions.
		$container.addClass('ec' + increment);
		increment++;

		if($container.length < 1){
			console.log('Image: ' + name + 'does not have the easyContainer parent!');
			return;
		}

		// Check for given element in given array.
		function checkPosition(pos, valid_options){
			if($.inArray(pos, valid_options) !== - 1){
				return true;
			}
			return false;
		}

		// Store valid crop positions here to set image positioning later on.
		var crop_positions = [];

		// Read the data set on the container, and decide whether to add it to the array or not.
		function checkCropPosition(position){
			var valid_options = ['top','bottom','left','right'];

			if($container.attr('data-crop-position-' + position)){
				var pos = $container.attr('data-crop-position-' + position);

				if(checkPosition(pos,valid_options)){
					crop_positions.push(pos);
					return;
				}

				console.log('Invalid crop position supplied!');
			}
		}

		// Vertical image crop.
		var cropVertical = function(){
			checkCropPosition('vertical');
			var amount = ($image.height() - $container.height()) / 2;

			// Check recalibrated image, if it's not received a decent crop, fix it.
			if($image.height() < $container.height()){
				$image.height($container.height()).width('auto');

				var new_horizontal_offset = ($image.width() - $container.width()) / 2;

				$image.css({
					'top': 0,
					'left': - new_horizontal_offset
				})
			}else{
				var top = - amount;

				if(checkPosition('top', crop_positions)){
					// Align image top.
					top = 0;
				}else if(checkPosition('bottom', crop_positions)){
					// Align image bottom.
					top = - ($image.height() - $container.height());
				}

				$image.css({
					'top': top
				})
			}
		}

		// Horizontal image crop.
		var cropHorizontal = function(){
			checkCropPosition('horizontal');

			var amount = ($image.width() - $container.width()) / 2;

			// Check recalibrated image, if it's not received a decent crop, fix it.
			if($image.width() < $container.width()){
				$image.width($container.width()).height('auto');

				var new_vertical_offset = ($image.height() - $container.height()) / 2;

				$image.css({
					'left': 0,
					'top': - new_vertical_offset
				})
			}else{
				var left = - amount;

				if(checkPosition('left', crop_positions)){
					// Align image left.
					left = 0;
				}else if(checkPosition('right', crop_positions)){
					// Align image right.
					left = - ($image.width() - $container.width());
				}

				$image.css({
					'left': left
				})
			}
		}

		// Resize the image in the appropriate dimension, then crop it accordingly
		//	depending on the aspect in which it's been resized.
		var init = function(){
			// Grab the offsets for comparison. This determines which aspect we will crop in.
			var horizontal_offset = $image.width() - $container.width();
			var vertical_offset = $image.height() - $container.height();

			if(horizontal_offset > vertical_offset){
				// We're going to crop vertically.
				if($image.height() < $container.height()){
					console.log('Image: ' + name + ' is too small to effectively crop. Pixellation commence!');
				}
				console.log('Image: ' + name + ' has been cropped vertically!');

				$image.height($container.height());
				cropHorizontal();
			}else{
				// We're going to crop horizontally.
				if($image.width() < $container.width()){
					console.log('Image: ' + name + ' is too narrow to effectively crop. Pixellation commence!');
				}
				console.log('Image: ' + name + ' has been cropped horizontally!');

				$image.width($container.width());
				cropVertical();
			}
		}

		// This calls other functions to perform the calculations.
		init();

		// This needs to work when aspect changes.
		if(settings.onResize){
			console.log('Recalculate crops on window resize!');
			$(window).resize(function(){
				init();
			})
		};

	});
};

$(window).load(function(){
	$('.easyCrop').easyCrop({
		onResize: true
	});
});
